"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
const vscode_languageserver_1 = require("vscode-languageserver");
const fs_ = require("fs");
const os_1 = require("os");
const fs = require("./fs");
const util = require("./util");
function newConnection(input, output, trace) {
    const reader = new vscode_jsonrpc_1.StreamMessageReader(input);
    let logger = null;
    if (trace.trace && trace.logfile) {
        try {
            logger = fs_.createWriteStream(trace.logfile, { flags: 'a', encoding: 'utf-8' });
        }
        catch (e) {
            console.error('Unable to initialize logger', e);
        }
    }
    const listen = reader.listen.bind(reader);
    reader.listen = (callback) => {
        const tracer = (message) => {
            doTrace(message, trace, logger, '-->');
            callback(message);
        };
        listen(tracer);
    };
    const writer = new vscode_jsonrpc_1.StreamMessageWriter(output);
    const write = writer.write.bind(writer);
    writer.write = (message) => {
        doTrace(message, trace, logger, '<--');
        write(message);
    };
    const connection = vscode_languageserver_1.createConnection(reader, writer);
    // Remove vscode-languageserver's stream listeners that kill the process if the stream is closed
    input.removeAllListeners('end');
    input.removeAllListeners('close');
    output.removeAllListeners('end');
    output.removeAllListeners('close');
    return connection;
}
exports.newConnection = newConnection;
function registerLanguageHandler(connection, strict, handler) {
    connection.onRequest('initialize', (params) => __awaiter(this, void 0, void 0, function* () {
        console.error('initialize', params);
        let remoteFs;
        if (strict) {
            remoteFs = new fs.RemoteFileSystem(connection);
        }
        else {
            remoteFs = new fs.LocalFileSystem(util.uri2path(params.rootPath));
        }
        try {
            return yield handler.initialize(params, remoteFs, strict);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onShutdown(() => {
        handler.shutdown().catch(e => {
            console.error('shutdown failed:', e);
        });
    });
    connection.onDidOpenTextDocument((params) => {
        handler.didOpen(params).catch(e => {
            console.error('textDocument/didOpen failed:', e);
        });
    });
    connection.onDidChangeTextDocument((params) => {
        handler.didChange(params).catch(e => {
            console.error('textDocument/didChange failed:', e);
        });
    });
    connection.onDidSaveTextDocument((params) => {
        handler.didSave(params).catch(e => {
            console.error('textDocument/didSave failed:', e);
        });
    });
    connection.onDidCloseTextDocument((params) => {
        handler.didClose(params).catch(e => {
            console.error('textDocument/didClose failed:', e);
        });
    });
    connection.onWorkspaceSymbol((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getWorkspaceSymbols(params);
            const exit = new Date().getTime();
            console.error('workspace/symbol', params.query, JSON.stringify(params.symbol), (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onDocumentSymbol((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getDocumentSymbol(params);
            const exit = new Date().getTime();
            console.error('textDocument/documentSymbol', params.textDocument.uri, (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onRequest('workspace/references', (params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getWorkspaceReference(params);
            const exit = new Date().getTime();
            console.error('workspace/xreferences', (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onDefinition((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getDefinition(params);
            const exit = new Date().getTime();
            console.error('definition', docid(params), (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onRequest('textDocument/xdefinition', (params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getXdefinition(params);
            const exit = new Date().getTime();
            console.error('xdefinition', docid(params), (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onHover((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getHover(params);
            const exit = new Date().getTime();
            console.error('hover', docid(params), (exit - enter) / 1000.0);
            return Promise.resolve(result);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onReferences((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getReferences(params);
            const exit = new Date().getTime();
            console.error('references', docid(params), 'found', result.length, (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(params, e);
            return Promise.reject(e);
        }
    }));
    connection.onRequest('workspace/xpackages', () => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getPackages();
            const exit = new Date().getTime();
            console.error('packages found', result.length, (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(e);
            return Promise.reject(e);
        }
    }));
    connection.onRequest('workspace/xdependencies', () => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getDependencies();
            const exit = new Date().getTime();
            console.error('dependencies found', result.length, (exit - enter) / 1000.0);
            return Promise.resolve(result || []);
        }
        catch (e) {
            console.error(e);
            return Promise.reject(e);
        }
    }));
    connection.onCompletion((params) => __awaiter(this, void 0, void 0, function* () {
        const enter = new Date().getTime();
        try {
            const result = yield handler.getCompletions(params);
            const exit = new Date().getTime();
            console.error('completion items found', result && result.items ? result.items.length : 0, (exit - enter) / 1000.0);
            return result;
        }
        catch (e) {
            console.error(e);
            throw e;
        }
    }));
}
exports.registerLanguageHandler = registerLanguageHandler;
function docid(params) {
    return params.textDocument.uri + ':' + params.position.line + ':' + params.position.character;
}
function dump(message) {
    return JSON.stringify(message);
}
function doTrace(message, options, stream, prefix) {
    if (options.trace) {
        const text = prefix + ' ' + dump(message);
        console.error(text);
        if (stream) {
            try {
                stream.write(text + os_1.EOL);
            }
            catch (e) {
            }
        }
    }
}
//# sourceMappingURL=connection.js.map